@@ -114,6 +114,8 @@ enum ofono_atom_type {
 	OFONO_ATOM_TYPE_GPRS = 16,
 	OFONO_ATOM_TYPE_GPRS_CONTEXT = 17,
 	OFONO_ATOM_TYPE_RADIO_SETTINGS = 18,
+	OFONO_ATOM_TYPE_STK = 19,
+	OFONO_ATOM_TYPE_NETTIME = 20,
 };
 
 enum ofono_atom_watch_condition {
@@ -172,8 +174,9 @@ void __ofono_atom_free(struct ofono_atom *atom);
 #include <ofono/radio-settings.h>
 
 #include <ofono/sim.h>
+#include <ofono/stk.h>
 
-void __ofono_cbs_sim_download(struct ofono_sim *sim,
+void __ofono_cbs_sim_download(struct ofono_stk *stk,
 				const guint8 *pdu, int pdu_len);
 
 #include <ofono/ssn.h>
@@ -262,3 +265,10 @@ struct sms;
 
 void __ofono_message_waiting_mwi(struct ofono_message_waiting *mw,
 				struct sms *sms, gboolean *out_discard);
+
+#include <ofono/nettime.h>
+
+void __ofono_nettime_probe_drivers(struct ofono_modem *modem);
+
+void __ofono_nettime_info_received(struct ofono_modem *modem,
+					struct ofono_network_time *info);
